/**
 * Drawing.java
 * 2/24/12
 */

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import javax.swing.JFrame;
import javax.swing.JPanel;

/**
 * Simple drawing program.
 * 
 * @author CPSC170A
 */
public class Drawing extends JPanel implements MouseListener,
        MouseMotionListener {

    /** Main panel width. */
    private static final Dimension PANEL_DIM = new Dimension(640, 480);

    /** Drawing panel width. */
    private static final Dimension DRAW_PANEL_DIM = new Dimension(584, 472);

    /** Width of lines for all tools. */
    private static final float LINE_WIDTH = 2.0f;

    /** List of objects to draw. */
    private ArrayList<Drawable> drawList = new ArrayList<Drawable>();

    /** Panel contain tool buttons. */
    private ToolPanel toolPanel;

    /**
     * Constructor, initializes window by creating a main panel, a left sub-
     * panel, and using this as the right sub panel.
     */
    public Drawing() {
        JFrame frame = new JFrame("Drawing");
        JPanel mainPanel = new JPanel();
        toolPanel = new ToolPanel();
        this.setBackground(Color.white);
        mainPanel.setBackground(Color.lightGray);
        mainPanel.setPreferredSize(PANEL_DIM);
        this.setPreferredSize(DRAW_PANEL_DIM);
        addMouseListener(this);
        addMouseMotionListener(this);
        mainPanel.add(toolPanel);
        mainPanel.add(this);
        frame.getContentPane().add(mainPanel);
        frame.pack();
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        frame.setResizable(false);
        frame.setVisible(true);
    }

    /**
     * Runs the application.
     * 
     * @param args command line argument
     */
    public static void main(final String[] args) {
        new Drawing();
    }

    /**
     * Draw to the right sub panel.
     * 
     * @param g graphics object to draw to
     */
    public final void paintComponent(final Graphics g) {
        super.paintComponent(g);
        Graphics2D g2 = (Graphics2D) g;
        g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                        RenderingHints.VALUE_ANTIALIAS_ON);
        g2.setStroke(new BasicStroke((float) LINE_WIDTH));
        for (int i = 0; i < drawList.size(); i++) {
            drawList.get(i).draw(g2);
        }
    }

    /**
     * Unused in this program.
     * 
     * @param event mouse event
     */
    public final void mouseClicked(final MouseEvent event) {}

    /**
     * Unused in this program.
     * 
     * @param event mouse event
     */
    public void mouseEntered(final MouseEvent event) {}

    /**
     * Unused in this program. 
     * 
     * @param event mouse event
     */
    public void mouseExited(final MouseEvent event) {}

    /**
     * Add appropriate object to the draw list based on selected tool.
     * 
     * @param event mouse event
     */
    public final void mousePressed(final MouseEvent event) {
	final int mouseX = event.getX();
	final int mouseY = event.getY();
        final Point2D.Double mousePoint = new Point2D.Double(mouseX, mouseY);
	if (toolPanel.isOvalSelected()) {
	    drawList.add(new Oval(mousePoint));
	} else if (toolPanel.isRectangleSelected()) {
	    drawList.add(new Rectangle(mousePoint));
	} else if (toolPanel.isLineSelected()) {
	    drawList.add(new Line(mousePoint));
	}
    }

    /**
     * Unused in this program.
     * 
     * @param event mouse event
     */
    public void mouseReleased(final MouseEvent event) {}

    /**
     * Resize the last added object.
     * 
     * @param event mouse event
     */
    public final void mouseDragged(final MouseEvent event) {
	final int mouseX = event.getX();
	final int mouseY = event.getY();
        final Point2D.Double mousePoint = new Point2D.Double(mouseX, mouseY);
	final Drawable drawableShape = drawList.getLast();
	drawableShape.updateWithPoint(mousePoint);
	repaint();
    }

    /**
     * Unused in this program.
     * 
     * @param event mouse event
     */
    public void mouseMoved(final MouseEvent event) { }

}
