/**
 * Shape.java
 * 2/24/12
 */

import java.awt.Graphics2D;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Point2D;

/**
 * Class that represents a drawable shape for the drawing program.
 * 
 * @author CPSC170
 */
public abstract class Shape implements Drawable {

    /** The opposite corners of the shape's bounding rectangle. */
    Point2D.Double startPoint, endPoint;

    /**
     * Constructor, initializes drawable shape with specified parameters.
     * 
     * @param initPoint1 first point of shape's bounding rectangle
     * @param initPoint2 second point of shape's bounding rectangle
     */
    public Shape(final Point2D.Double initPoint) {
        startPoint = initPoint;
	endPoint = initPoint;
    }

    /**
     * Updates this shape's end point to be the specified point.
     * 
     * @param point the new end point
     */
    public void updateWithPoint(final Point2D.Double point) {
	endPoint = point;
    }

    /**
     * Draw this shape.
     * 
     * @param g2 graphics object to draw to
     */
    public void draw(final Graphics2D g2) {
        double x, y, width, height;
        if (startPoint.x < endPoint.x) {
            x = startPoint.x;
            width = (endPoint.x - startPoint.x);
        } else {
            x = endPoint.x;
            width = (startPoint.x - endPoint.x);
        }
        if (startPoint.y < endPoint.y) {
            y = startPoint.y;
            height = (endPoint.y - startPoint.y);
        } else {
            y = endPoint.y;
            height = (startPoint.y - endPoint.y);
        }
	draw(g2, x, y, width, height);
    }

    /**
     * Draw this shape with with the specified location and size.
     *
     * @param g2 graphics object to draw to
     * @param x x coordinate of shape location
     * @param y y coordinate of shape location
     * @param width width of shape
     * @param height height of shape
     */
    protected abstract void draw(final Graphics2D g2,
				 final double x, final double y,
				 final double width, final double height);

}
