/**
 * ToolPanel.java
 * 2/24/12
 */

import java.awt.Color;
import java.awt.Dimension;
import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JPanel;
import javax.swing.JRadioButton;

/**
 * Panel containing drawing tool buttons.
 * 
 * @author CPSC170
 */
public class ToolPanel extends JPanel {

    /** Array of tool radio buttons. */
    private JRadioButton[] buttons;

    /** Array of button icon file names. */
    private static String[] icons = {"lightoval.png",
				     "lightrect.png",
				     "lightline.png"};

    /** Array of selected button icon file names. */
    private static String[] iconsSelected = {"darkoval.png",
					     "darkrect.png",
					     "darkline.png"};

    /** Index of oval button. */
    public static final int OVAL = 0;

    /** Index of line button. */
    public static final int RECTANGLE = 1;

    /** Index of pencil button. */
    public static final int LINE = 2;

    /** Left sub panel width. */
    private static final Dimension PANEL_DIM = new Dimension(40, 472);

    /**
     * Constructor, initialize panel with tool buttons.
     */
    public ToolPanel() {
        ButtonGroup group = new ButtonGroup();
        buttons = new JRadioButton[icons.length];
        for (int i = 0; i < buttons.length; i++) {
            buttons[i] = new JRadioButton(new ImageIcon(icons[i]));
            buttons[i].setSelectedIcon(new ImageIcon(iconsSelected[i]));
            buttons[i].setBackground(Color.lightGray);
            group.add(buttons[i]);
            add(buttons[i]);
        }
        buttons[0].setSelected(true);
        setBackground(Color.lightGray);
        setPreferredSize(PANEL_DIM);
    }

    /**
     * Returns whether the specified button is currently selected.
     * 
     * @param toolIndex index of the button to check if selected
     * @throws IndexOutOfBoundsException if toolIndex < 0 or >= the number
     * of buttons
     * @return true if the specified button is selected, false otherwise
     */
    public final boolean isSelected(final int toolIndex) {
	if (toolIndex < 0 || toolIndex >= buttons.length) {
	    String errorMsg = "Tool index must be 0 - " + (buttons.length - 1);
	    throw new IndexOutOfBoundsException(errorMsg);
	}
	return buttons[toolIndex].isSelected();
    }

    /**
     * Returns whether the oval button is currently selected.
     * 
     * @return true if the oval button is selected, false otherwise
     */
    public final boolean isOvalSelected() {
	return isSelected(OVAL);
    }

    /**
     * Returns whether the rectangle button is currently selected.
     * 
     * @return true if the rectangle button is selected, false otherwise
     */
    public final boolean isRectangleSelected() {
	return isSelected(RECTANGLE);
    }

    /**
     * Returns whether the line button is currently selected.
     * 
     * @return true if the line button is selected, false otherwise
     */
    public final boolean isLineSelected() {
	return isSelected(LINE);
    }
}
